@@ -1,10 +1,10 @@
 #!/usr/bin/perl
 
-# $Id: ip_relay.pl,v 1.10 2000/01/05 23:57:28 gavin Exp $
+# $Id: ip_relay.pl,v 1.13 2000/05/27 07:37:19 gavin Exp $
 # 
 # ip_relay.pl
 #
-# Copyright (C) 1999 Gavin Stewart
+# Copyright (C) 1999,2000 Gavin Stewart
 #
 
 # ip_relay
@@ -22,6 +22,7 @@ use strict;
 use POSIX;
 use Socket;
 use FileHandle;
+use Getopt::Std;
 
 # ** Default settings for variables setable within shell.
 
@@ -44,11 +45,12 @@ my (@all_vars);
 
 # ** Default settings for non-setable variables.
 my $app_name = "ip_relay.pl";	#Application name.
-my $version = "0.7";		#Application version number.
+my $version = "0.71";		#Application version number.
 my $max_listen_bind_attempts = 20; #Max no. attempts to bind to local_addrs
 				     #and local_port.
 my $forward_select_time = 0.01;    #At least some delay needed.
 my $qlen = 5;		#how long to let the connect queue grow.
+my $daemonise = 0;	#gets set from command line, no console, no output.
 
 # ** Other global vars.
 my $time_now;		#Variable containing the "currentish" time.
@@ -67,13 +69,34 @@ $SIG{QUIT} = \&cleanup_handler;
 
 if ($#ARGV >= 0) {  #someone used command line "fast setup".
     if (! &parse_param) {
-        print "Usage: $0 [local_port:remote_host:remote_port]\n";
+        print "Usage: $0 [ [-d] [-b n] local_port:remote_host:remote_port]\n";
+	print "    -d        Daemon mode, go straight into background.\n";
+	print "              (you loose all logging and console access.)\n";
+	print "    -b n      Bandwidth, where n is max bytes/sec.\n";
 	exit (1);
     } 
 }
 
 &print_version;
 
+if ($daemonise) {
+    my $child_id = fork();
+    if (! defined ($child_id)) {
+        die ("Fork failed...die-ing: $!\n");
+    } else {
+    	if (! $child_id) {
+	    #child
+	    close (STDIN);
+	    close (STDOUT);	#should we open this to /dev/null ?
+	    close (STDERR);
+	    #POSIX::setsid();
+	} else {
+	    #parent
+	    exit (0);	#Succesful fork of child, parent work is completed.
+	}
+    }
+}
+
 fcntl(STDIN, F_SETFL, O_NONBLOCK); #dont make our STDIN "block"
 print "> ";
 $last_skew=time;		#I have to init this someplace.
@@ -93,11 +116,20 @@ exit (0);
 
 sub print_version {
     print STDERR "\n$app_name Version: $version\n";
-    print STDERR "Copyright (C) 1999 Gavin Stewart\n\n";
+    print STDERR "Copyright (C) 1999,2000 Gavin Stewart\n\n";
 }
 
 sub parse_param {
-   #print "$ARGV[0]\n";
+   use vars qw/ $opt_d $opt_b /;	#For getopts.
+   #print "Args: ".join(",", @ARGV)."\n";
+   getopts('db:');
+   #print "opt_d: $opt_d\n";
+   #print "opt_b: $opt_b\n";
+   $daemonise = 1 if ($opt_d);
+   $main::bandwidth = $opt_b if ($opt_b);
+   #print "Args: ".join(",", @ARGV)."\n";
+
+   #After getopts, we expect to just have our "quick" command line.
    if ($ARGV[0] =~ /(\d+):([^:]+):(\d+)/) {
 	my ($local_port, $remote_addrs, $remote_port)=($1,$2,$3);
  	      
@@ -116,9 +148,13 @@ sub parse_param {
         print STDERR "  local_port\t$local_port\n";
         print STDERR "  remote_addrs\t$remote_addrs\n";
         print STDERR "  remote_port\t$remote_port\n";
+	print STDERR "  bandwidth\t$main::bandwidth\n";
 	print STDERR "  forwarder 99 set.\n\n";
 	return (1);
    } else {
+        if ($daemonise || $main::bandwidth) {
+	    print "\nIt only makes sense to use -d and -b with local_port:remote_host:remote_port !\n\n";
+	}
 	return (0);
    }
 }
@@ -126,6 +162,9 @@ sub parse_param {
 sub check_user_input {
     #We want to see if the user types anything, and effect any variable
     #changes also.
+
+    return if ($daemonise);	#We are not connected to the console.
+
     my ($input, $cmd, $variable, @value, $var_name);
 
     $input = <STDIN>;
